==> style-o-matic-1.1.3.tgz: package/es/lib/propUtilities.js <==
var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

import _ from "lodash";
import PropTypes from "prop-types";
import elementType from "prop-types-extra/lib/elementType";
import isRequiredForA11y from "prop-types-extra/lib/isRequiredForA11y";
import cx from "classnames";
import { getParentClass } from "./ROLE.js";
import { uID } from "./coreUtilities.js";
import UIGlobals from "./UIGlobals.js";

var OBSERVABLE_EVENTS = ["scroll", "wheel", "mousemove", "keypress", "copy", "paste", "touchmove", "touchstart", "error", "load", "toggle"];

var NATIVE_PROPS = ["id", "name", "type", "value", "checked", "required", "className", "role", "style", "href", "src", "width", "height", "colSpan", "children", "onClick", "onSelect", "onChange", "onBlur", "onFocus", "onMouseDown", "onMouseUp", "onMouseEnter", "onMouseLeave", "onKeyDown", "onKeyUp", "onToggle", "onSubmit"];

var INHERITED_PROPS = ["uiclass", "path", "disabled", "open", "uidata", "invalid"];

var DefaultPropTypes = {
  uuid: PropTypes.string,
  id: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  name: PropTypes.string,
  type: PropTypes.string,
  value: PropTypes.string,
  label: PropTypes.string,
  invalid: PropTypes.bool,
  checked: PropTypes.bool,
  required: PropTypes.bool,
  inset: PropTypes.bool,
  raised: PropTypes.bool,
  reveal: PropTypes.bool,
  well: PropTypes.bool,
  panel: PropTypes.bool,
  rounded: PropTypes.bool,
  circular: PropTypes.bool,
  masked: PropTypes.bool,
  renderAs: elementType,
  className: PropTypes.string,
  role: PropTypes.string,
  group: PropTypes.string,
  tooltip: PropTypes.string,
  children: PropTypes.node,
  uiclass: PropTypes.string,
  uirole: PropTypes.string,
  path: PropTypes.string,
  text: PropTypes.string,
  icon: PropTypes.string,
  to: PropTypes.string,
  disabled: PropTypes.bool,
  open: PropTypes.bool,
  active: PropTypes.bool,
  overlay: PropTypes.bool,
  fixed: PropTypes.bool,
  color: PropTypes.oneOf(["transparent", "white", "grey", "black", "red", "orange", "yellow", "green", "blue", "indigo", "violet", "!white", "!grey", "!black", "!red", "!orange", "!yellow", "!green", "!blue", "!indigo", "!violet"]),
  colorStyle: PropTypes.oneOf(["fill", "outline", "text"]),
  colorHover: PropTypes.bool,
  displaySize: PropTypes.oneOf(["smallest", "small", "medium", "large", "largest"]),
  orientation: PropTypes.oneOf(["vertical", "horizontal"]),
  textAlign: PropTypes.oneOf(["left", "right", "center"]),
  contentAlign: PropTypes.oneOf(["left", "right", "center"]),
  attach: PropTypes.oneOf(["left", "right", "top", "bottom"]),
  position: PropTypes.oneOf(["left", "before", "right", "after", "top", "above", "bottom", "below"]),
  observe: PropTypes.oneOf(OBSERVABLE_EVENTS),
  dispatch: PropTypes.oneOf(OBSERVABLE_EVENTS),
  uidata: PropTypes.object
};

var DefaultPropValues = {
  uuid: null,
  renderAs: "div",
  className: null,
  role: null,
  name: null,
  type: null,
  label: null,
  value: null,
  invalid: null,
  checked: null,
  required: null,
  group: null,
  tooltip: null,
  children: null,
  uiclass: null,
  uirole: "",
  path: "/",
  attach: null,
  text: null,
  icon: null,
  inset: null,
  raised: null,
  reveal: null,
  well: null,
  panel: null,
  rounded: null,
  circular: null,
  masked: null,
  to: null,
  disabled: false,
  open: false,
  active: false,
  overlay: null,
  fixed: null,
  color: null,
  colorStyle: null,
  colorHover: null,
  displaySize: null,
  orientation: null,
  textAlign: null,
  contentAlign: null,
  position: null,
  observe: null,
  dispatch: null,
  uidata: {}
};

function setPropTypes() {
  var isA11y = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var config = arguments[1];
  var uidataConfig = arguments[2];

  var obj = _.clone(DefaultPropTypes);
  if (isA11y) {
    obj.id = isRequiredForA11y(PropTypes.oneOfType([PropTypes.string, PropTypes.number]));
  }
  if (typeof config !== "undefined" && config !== null) {
    Object.entries(config).forEach(function (_ref) {
      var attr = _ref[0],
          propString = _ref[1];

      var required = String(propString).indexOf("!") !== -1;
      var prop = String(propString).replace("!", "");
      if (typeof PropTypes[prop] !== "undefined") {
        obj[attr] = required ? PropTypes[prop].isRequired : PropTypes[prop];
      }
    });
  }
  if (typeof uidataConfig !== "undefined" && uidataConfig !== null) {
    var uidata = {};
    Object.entries(uidataConfig).forEach(function (_ref2) {
      var attr = _ref2[0],
          propString = _ref2[1];

      var required = String(propString).indexOf("!") !== -1;
      var prop = String(propString).replace("!", "");
      if (typeof PropTypes[prop] !== "undefined") {
        uidata[attr] = required ? PropTypes[prop].isRequired : PropTypes[prop];
      }
    });
    obj.uidata = PropTypes.shape(uidata);
  }
  return obj;
}

export function getCorePropTypes(config, uidataConfig) {
  return setPropTypes(false, config, uidataConfig);
}

export function getPropTypesA11y(config, uidataConfig) {
  return setPropTypes(true, config, uidataConfig);
}

function setPropDefaults(autoId, config, uidataConfig) {
  var obj = _.clone(DefaultPropValues);
  obj.uuid = uID();
  if (typeof config !== "undefined" && config !== null) {
    Object.entries(config).forEach(function (_ref3) {
      var attr = _ref3[0],
          prop = _ref3[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        obj[attr] = _.clone(prop);
      } else {
        obj[attr] = prop;
      }
    });
  }
  if (typeof uidataConfig !== "undefined" && uidataConfig !== null) {
    var d = {};
    Object.entries(uidataConfig).forEach(function (_ref4) {
      var attr = _ref4[0],
          prop = _ref4[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        d[attr] = _.clone(prop);
      } else {
        d[attr] = prop;
      }
    });
    obj.uidata = d;
    obj.privatedata = d;
  }
  if (_.isNil(obj.uiclass) && !_.isNil(obj.uirole)) {
    obj.uiclass = obj.uirole;
  }
  if (autoId && _.isNil(obj.id)) {
    obj.id = obj.uirole + "_" + obj.uuid;
  }
  return obj;
}

export function getCorePropDefaults(config, uidataConfig) {
  return setPropDefaults(false, config, uidataConfig);
}

export function getPropDefaultsAutoId(config, uidataConfig) {
  return setPropDefaults(true, config, uidataConfig);
}

export function getUIClassString(props, state) {
  var _coreClasses, _styleClasses;

  var uiclass = props.uiclass,
      className = props.className,
      overlay = props.overlay,
      fixed = props.fixed,
      inset = props.inset,
      raised = props.raised,
      reveal = props.reveal,
      well = props.well,
      panel = props.panel,
      rounded = props.rounded,
      circular = props.circular,
      masked = props.masked,
      color = props.color,
      colorStyle = props.colorStyle,
      colorHover = props.colorHover,
      orientation = props.orientation,
      textAlign = props.textAlign,
      contentAlign = props.contentAlign,
      position = props.position;


  var disabled = state && state.disabled ? state.disabled : props.disabled;
  var active = state && state.active ? state.active : props.active;
  var open = state && state.open ? state.open : props.open;
  var invalid = state && state.invalid ? state.invalid : props.invalid;

  var sUIclass = "ui-" + uiclass;
  var hasUIclass = !className || className.indexOf(sUIclass) === -1;
  var coreClasses = (_coreClasses = {}, _coreClasses[sUIclass] = hasUIclass, _coreClasses["ui-position-" + position] = position, _coreClasses["ui-orientation-" + orientation] = orientation, _coreClasses["ui-content-align-" + contentAlign] = contentAlign, _coreClasses["ui-text-align-" + textAlign] = textAlign, _coreClasses["ui-overlay"] = overlay, _coreClasses["ui-fixed"] = fixed, _coreClasses["ui-inset"] = inset, _coreClasses["ui-raised"] = raised, _coreClasses["ui-reveal"] = reveal, _coreClasses["ui-rounded"] = rounded, _coreClasses["ui-circular"] = circular, _coreClasses["ui-well"] = well, _coreClasses["ui-panel"] = panel, _coreClasses["ui-masked"] = masked, _coreClasses.disabled = disabled, _coreClasses.active = active, _coreClasses.open = open, _coreClasses.invalid = invalid, _coreClasses);

  var theme = UIGlobals.readSetting("theme");
  var styleClasses = (_styleClasses = {}, _styleClasses[sUIclass + "-theme-" + theme] = uiclass && theme, _styleClasses);

  if (!_.isNil(color)) {
    var nohover = String(color).indexOf("!") !== -1;
    var cleanColor = String(color).replace("!", "");
    var colorClass = "ui-" + cleanColor;
    if (!_.isNil(colorStyle)) {
      colorClass += "-" + colorStyle;
    }
    if (nohover) {
      colorClass += "-no-hover";
    } else if (!_.isNil(colorHover)) {
      colorClass += "-hover";
    }
    styleClasses[colorClass] = true;
  }
  // if (!_.isNil(orientation)) {
  //   classes[`ui-orientation-${orientation}`] = true;
  // }
  // if (!_.isNil(position)) {
  //   classes[`ui-position-${position}`] = true;
  // }
  // if (!_.isNil(contentAlign)) {
  //   classes[`ui-content-align-${contentAlign}`] = true;
  // }
  // if (!_.isNil(textAlign)) {
  //   classes[`ui-text-align-${textAlign}`] = true;
  // }
  var uiClassCore = cx(className, coreClasses);
  var uiClassStyle = cx("", styleClasses);
  var uiClassFull = cx(uiClassCore, uiClassStyle);
  return [uiClassFull, uiClassCore, uiClassStyle];
}

export function getValidProps(source, state) {
  var obj = { props: {}, inherited: {} };
  if (source.style) {
    var uiStyle = {};
    Object.entries(source.style).forEach(function (_ref5) {
      var attr = _ref5[0],
          prop = _ref5[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        uiStyle[attr] = _.clone(prop);
      } else {
        uiStyle[attr] = prop;
      }
    });
    obj.props.style = uiStyle;
  }
  Object.entries(source).forEach(function (_ref6) {
    var attr = _ref6[0],
        value = _ref6[1];

    if (attr !== "style") {
      if (NATIVE_PROPS.indexOf(attr) !== -1 && value !== null) {
        obj[attr] = value;
        obj.props[attr] = value;
      } else if (attr !== "uidata" && attr !== "privatedata") {
        if ((typeof value === "undefined" ? "undefined" : _typeof(value)) === "object") {
          obj[attr] = _.clone(value);
        } else {
          obj[attr] = value;
        }
      }
      if (INHERITED_PROPS.indexOf(attr) !== -1 && !_.isNil(value)) {
        obj.inherited[attr] = value;
      }
    }
  });
  if (typeof source.privatedata !== "undefined") {
    obj.uidata = _.clone(source.privatedata, source.uidata);
  }
  var uiClassNames = getUIClassString(source, state);
  obj.className = uiClassNames[0];
  obj.coreClassName = uiClassNames[1];
  obj.styleClassName = uiClassNames[2];
  obj.props.className = uiClassNames[0];
  if (_.isNil(obj.parentclass)) {
    obj.parentclass = getParentClass(obj);
  }
  obj.inherited.parentclass = obj.parentclass;
  if (_.isNil(obj.inherited.uiclass)) {
    obj.inherited.uiclass = "";
  }
  return obj;
}

==> style-o-matic-1.1.5.tgz: package/es/lib/propUtilities.js <==
var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

import _ from "lodash";
import PropTypes from "prop-types";
import elementType from "prop-types-extra/lib/elementType";
import isRequiredForA11y from "prop-types-extra/lib/isRequiredForA11y";
import cx from "classnames";
import { getParentClass } from "./ROLE.js";
import { uID } from "./coreUtilities.js";
import UIGlobals from "./UIGlobals.js";

var OBSERVABLE_EVENTS = ["scroll", "wheel", "mousemove", "keypress", "copy", "paste", "touchmove", "touchstart", "error", "load", "toggle"];

var NATIVE_PROPS = ["id", "name", "type", "value", "checked", "defaultChecked", "required", "className", "role", "style", "href", "target", "src", "width", "height", "colSpan", "children", "onClick", "onSelect", "onChange", "onBlur", "onFocus", "onMouseDown", "onMouseUp", "onMouseEnter", "onMouseLeave", "onKeyDown", "onKeyUp", "onToggle", "onSubmit"];

var INHERITED_PROPS = ["uiclass", "path", "disabled", "uidata", "invalid"];

var DefaultPropTypes = {
  uuid: PropTypes.string,
  id: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  name: PropTypes.string,
  type: PropTypes.string,
  value: PropTypes.string,
  label: PropTypes.string,
  invalid: PropTypes.bool,
  checked: PropTypes.bool,
  defaultChecked: PropTypes.bool,
  required: PropTypes.bool,
  inset: PropTypes.bool,
  raised: PropTypes.bool,
  reveal: PropTypes.bool,
  well: PropTypes.bool,
  panel: PropTypes.bool,
  rounded: PropTypes.bool,
  circular: PropTypes.bool,
  masked: PropTypes.bool,
  renderAs: elementType,
  className: PropTypes.string,
  role: PropTypes.string,
  group: PropTypes.string,
  tooltip: PropTypes.string,
  children: PropTypes.node,
  uiclass: PropTypes.string,
  uirole: PropTypes.string,
  path: PropTypes.string,
  text: PropTypes.string,
  icon: PropTypes.string,
  to: PropTypes.string,
  disabled: PropTypes.bool,
  open: PropTypes.bool,
  active: PropTypes.bool,
  visible: PropTypes.bool,
  overlay: PropTypes.bool,
  fixed: PropTypes.bool,
  color: PropTypes.oneOf(["transparent", "white", "grey", "black", "red", "orange", "yellow", "green", "blue", "indigo", "violet", "!white", "!grey", "!black", "!red", "!orange", "!yellow", "!green", "!blue", "!indigo", "!violet"]),
  colorStyle: PropTypes.oneOf(["fill", "outline", "text"]),
  colorHover: PropTypes.bool,
  displaySize: PropTypes.oneOf(["smallest", "small", "medium", "large", "largest"]),
  orientation: PropTypes.oneOf(["vertical", "horizontal"]),
  textAlign: PropTypes.oneOf(["left", "right", "center"]),
  contentAlign: PropTypes.oneOf(["left", "right", "center"]),
  position: PropTypes.oneOf(["left", "before", "right", "after", "top", "above", "bottom", "below"]),
  shadow: PropTypes.string,
  observe: PropTypes.oneOf(OBSERVABLE_EVENTS),
  dispatch: PropTypes.oneOf(OBSERVABLE_EVENTS),
  uidata: PropTypes.object
};

var DefaultPropValues = {
  uuid: null,
  renderAs: "div",
  className: null,
  role: null,
  name: null,
  type: null,
  label: null,
  value: null,
  invalid: null,
  checked: null,
  defaultChecked: null,
  required: null,
  group: null,
  tooltip: null,
  children: null,
  uiclass: null,
  uirole: "",
  path: "/",
  text: null,
  icon: null,
  inset: null,
  raised: null,
  reveal: null,
  well: null,
  panel: null,
  rounded: null,
  circular: null,
  masked: null,
  to: null,
  disabled: null,
  open: null,
  active: null,
  visible: null,
  overlay: null,
  fixed: null,
  color: null,
  colorStyle: null,
  colorHover: null,
  displaySize: null,
  orientation: null,
  textAlign: null,
  contentAlign: null,
  position: null,
  shadow: null,
  observe: null,
  dispatch: null,
  uidata: {}
};

function stringToPropType(propString) {
  var required = String(propString).indexOf("!") !== -1;
  var prop = String(propString).replace("!", "");
  if (typeof PropTypes[prop] !== "undefined") {
    return required ? PropTypes[prop].isRequired : PropTypes[prop];
  }
  return PropTypes.string;
}

function assignPropType(propVal) {
  if (_.isArray(propVal)) {
    var typeValues = propVal.map(function (val) {
      return stringToPropType(val);
    });
    return PropTypes.oneOfType(typeValues);
  }
  return stringToPropType(propVal);
}

function setPropTypes() {
  var isA11y = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var config = arguments[1];
  var uidataConfig = arguments[2];

  var obj = _.clone(DefaultPropTypes);
  if (isA11y) {
    obj.id = isRequiredForA11y(PropTypes.oneOfType([PropTypes.string, PropTypes.number]));
  }
  if (typeof config !== "undefined" && config !== null) {
    Object.entries(config).forEach(function (_ref) {
      var attr = _ref[0],
          val = _ref[1];

      obj[attr] = assignPropType(val);
    });
  }
  if (typeof uidataConfig !== "undefined" && uidataConfig !== null) {
    var uidata = {};
    Object.entries(uidataConfig).forEach(function (_ref2) {
      var attr = _ref2[0],
          val = _ref2[1];

      uidata[attr] = assignPropType(val);
    });
    obj.uidata = PropTypes.shape(uidata);
  }
  return obj;
}

export function setCorePropTypes(config, uidataConfig) {
  return setPropTypes(false, config, uidataConfig);
}

export function setPropTypesA11y(config, uidataConfig) {
  return setPropTypes(true, config, uidataConfig);
}

function setPropDefaults(autoId, config, uidataConfig) {
  var obj = _.clone(DefaultPropValues);
  obj.uuid = uID();
  if (typeof config !== "undefined" && config !== null) {
    Object.entries(config).forEach(function (_ref3) {
      var attr = _ref3[0],
          prop = _ref3[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        obj[attr] = _.clone(prop);
      } else {
        obj[attr] = prop;
      }
    });
  }
  if (typeof uidataConfig !== "undefined" && uidataConfig !== null) {
    var d = {};
    Object.entries(uidataConfig).forEach(function (_ref4) {
      var attr = _ref4[0],
          prop = _ref4[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        d[attr] = _.clone(prop);
      } else {
        d[attr] = prop;
      }
    });
    obj.uidata = d;
    obj.privatedata = d;
  }
  if (_.isNil(obj.uiclass) && !_.isNil(obj.uirole)) {
    obj.uiclass = obj.uirole;
  }
  if (autoId && _.isNil(obj.id)) {
    obj.id = obj.uirole + "_" + obj.uuid;
  }
  return obj;
}

export function setCorePropDefaults(config, uidataConfig) {
  return setPropDefaults(false, config, uidataConfig);
}

export function setPropDefaultsAutoId(config, uidataConfig) {
  return setPropDefaults(true, config, uidataConfig);
}

export function getUIClassString(props, state) {
  var _coreClasses, _styleClasses;

  var uiclass = props.uiclass,
      className = props.className,
      overlay = props.overlay,
      fixed = props.fixed,
      inset = props.inset,
      raised = props.raised,
      reveal = props.reveal,
      well = props.well,
      panel = props.panel,
      rounded = props.rounded,
      circular = props.circular,
      masked = props.masked,
      color = props.color,
      colorStyle = props.colorStyle,
      colorHover = props.colorHover,
      textAlign = props.textAlign,
      contentAlign = props.contentAlign,
      position = props.position,
      shadow = props.shadow;


  var disabled = state && !_.isNil(state.disabled) ? state.disabled : props.disabled;
  var active = state && !_.isNil(state.active) ? state.active : props.active;
  var open = state && !_.isNil(state.open) ? state.open : props.open;
  var visible = state && !_.isNil(state.visible) ? state.visible : props.visible;
  var invalid = state && !_.isNil(state.invalid) ? state.invalid : props.invalid;
  var orientation = state && state.orientation ? state.orientation : props.orientation;

  var sUIclass = "ui-" + uiclass;
  var hasUIclass = !className || className.indexOf(sUIclass) === -1;
  var coreClasses = (_coreClasses = {}, _coreClasses[sUIclass] = hasUIclass, _coreClasses["ui-position-" + position] = position, _coreClasses["ui-orientation-" + orientation] = orientation, _coreClasses["ui-content-align-" + contentAlign] = contentAlign, _coreClasses["ui-text-align-" + textAlign] = textAlign, _coreClasses["ui-overlay"] = overlay, _coreClasses["ui-fixed"] = fixed, _coreClasses["ui-inset"] = inset, _coreClasses["ui-raised"] = raised, _coreClasses["ui-reveal"] = reveal, _coreClasses["ui-rounded"] = rounded, _coreClasses["ui-circular"] = circular, _coreClasses["ui-well"] = well, _coreClasses["ui-panel"] = panel, _coreClasses["ui-masked"] = masked, _coreClasses["ui-shadow"] = shadow, _coreClasses.disabled = disabled, _coreClasses.active = active, _coreClasses.open = open, _coreClasses.visible = visible, _coreClasses.invalid = invalid, _coreClasses);

  var theme = UIGlobals.readSetting("theme");
  var styleClasses = (_styleClasses = {}, _styleClasses[sUIclass + "-theme-" + theme] = uiclass && theme, _styleClasses);

  if (!_.isNil(color)) {
    var nohover = String(color).indexOf("!") !== -1;
    var cleanColor = String(color).replace("!", "");
    var colorClass = "ui-" + cleanColor;
    if (!_.isNil(colorStyle)) {
      colorClass += "-" + colorStyle;
    }
    if (nohover) {
      colorClass += "-no-hover";
    } else if (!_.isNil(colorHover)) {
      colorClass += "-hover";
    }
    styleClasses[colorClass] = true;
  }
  // if (!_.isNil(orientation)) {
  //   classes[`ui-orientation-${orientation}`] = true;
  // }
  // if (!_.isNil(position)) {
  //   classes[`ui-position-${position}`] = true;
  // }
  // if (!_.isNil(contentAlign)) {
  //   classes[`ui-content-align-${contentAlign}`] = true;
  // }
  // if (!_.isNil(textAlign)) {
  //   classes[`ui-text-align-${textAlign}`] = true;
  // }
  var uiClassCore = cx(className, coreClasses);
  var uiClassStyle = cx("", styleClasses);
  var uiClassFull = cx(uiClassCore, uiClassStyle);
  return [uiClassFull, uiClassCore, uiClassStyle];
}

export function getValidProps(source, state) {
  var obj = { props: {}, inherited: {} };
  if (source.style) {
    var uiStyle = {};
    Object.entries(source.style).forEach(function (_ref5) {
      var attr = _ref5[0],
          prop = _ref5[1];

      if ((typeof prop === "undefined" ? "undefined" : _typeof(prop)) === "object") {
        uiStyle[attr] = _.clone(prop);
      } else {
        uiStyle[attr] = prop;
      }
    });
    obj.props.style = uiStyle;
  }
  Object.entries(source).forEach(function (_ref6) {
    var attr = _ref6[0],
        value = _ref6[1];

    if (attr !== "style") {
      if (NATIVE_PROPS.indexOf(attr) !== -1 && value !== null) {
        obj[attr] = value;
        obj.props[attr] = value;
      } else if (attr !== "uidata" && attr !== "privatedata") {
        if ((typeof value === "undefined" ? "undefined" : _typeof(value)) === "object") {
          obj[attr] = _.clone(value);
        } else {
          obj[attr] = value;
        }
      }
      if (INHERITED_PROPS.indexOf(attr) !== -1 && !_.isNil(value)) {
        obj.inherited[attr] = value;
      }
    }
  });
  if (typeof source.privatedata !== "undefined") {
    obj.uidata = _.clone(source.privatedata, source.uidata);
  }
  var uiClassNames = getUIClassString(source, state);
  obj.className = uiClassNames[0];
  obj.coreClassName = uiClassNames[1];
  obj.styleClassName = uiClassNames[2];
  obj.props.className = uiClassNames[0];
  if (_.isNil(obj.parentclass)) {
    obj.parentclass = getParentClass(obj);
  }
  obj.inherited.parentclass = obj.parentclass;
  if (_.isNil(obj.inherited.uiclass)) {
    obj.inherited.uiclass = "";
  }
  return obj;
}
